"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = _default;

var _helperModuleTransforms = require("@babel/helper-module-transforms");

// @ts-expect-error missing typedef
var IGNORED_MODULES = ['react'];

function _default(_ref) {
  var t = _ref.types;
  var affectedParents = {};

  var createRequireExpression = function createRequireExpression(programPath, info) {
    var requireExpression = t.callExpression(t.identifier('require'), [t.stringLiteral(info.source)]);

    if (info.property === null) {
      return (0, _helperModuleTransforms.wrapInterop)(programPath, requireExpression, 'namespace');
    }

    return t.memberExpression(info.property === 'default' ? (0, _helperModuleTransforms.wrapInterop)(programPath, requireExpression, 'default') : requireExpression, t.identifier(info.property));
  };

  var createConstRequireExpression = function createConstRequireExpression(programPath, info) {
    return t.variableDeclaration('const', [t.variableDeclarator(t.identifier(info.local), createRequireExpression(programPath, info))]);
  };

  var insertDeclaration = function insertDeclaration(path, programPath, info) {
    var parentWithBody = path.findParent(function (p) {
      return (// @ts-ignore looking for a node with body
        p.node !== undefined && p.node.body !== undefined && p.node.body.length
      );
    });

    if (!affectedParents[info.local]) {
      affectedParents[info.local] = new WeakSet();
    }

    if (!affectedParents[info.local].has(parentWithBody) && !parentWithBody.scope.hasBinding(info.local)) {
      parentWithBody.unshiftContainer( // @ts-expect-error wrong typedef type
      'body', createConstRequireExpression(programPath, info));
      affectedParents[info.local].add(parentWithBody);
    }
  };

  return {
    inherits: require('babel-plugin-syntax-jsx'),
    visitor: {
      ImportDeclaration: function (_ImportDeclaration) {
        function ImportDeclaration(_x) {
          return _ImportDeclaration.apply(this, arguments);
        }

        ImportDeclaration.toString = function () {
          return _ImportDeclaration.toString();
        };

        return ImportDeclaration;
      }(function (path) {
        var specifiers = path.node.specifiers;
        var programPath = path.parentPath;

        if (IGNORED_MODULES.includes(path.node.source.value)) {
          return;
        }

        var hasReplaced = false;
        specifiers.forEach(function (specifier) {
          var info = specifier.type === 'ImportDefaultSpecifier' ? {
            local: specifier.local.name,
            property: 'default',
            source: path.node.source.value
          } : specifier.type === 'ImportSpecifier' ? {
            local: specifier.local.name,
            property: specifier.imported.name,
            source: path.node.source.value
          } : {
            local: specifier.local.name,
            property: null,
            source: path.node.source.value
          };
          var binding = path.scope.getBinding(info.local);
          if (!binding) return;
          path.scope.rename(info.local, "____________".concat(info.local, "____________"));
          binding.referencePaths.forEach(function (referencePath) {
            insertDeclaration(referencePath, programPath, info);
            hasReplaced = true;
          });
          path.scope.rename("____________".concat(info.local, "____________"), info.local);
        });
        if (hasReplaced) path.remove();
      })
    }
  };
}